import {ActionContext} from "vuex";
import * as seed from "@/seed";
import BilliDB from "@/indexdDB";
import { v4 as uuidv4 } from 'uuid';
import {SEED_VERSION} from "@/seed";
import NameKey from "@/store/classes/NameKey";
import InventoryItem from "@/store/classes/InventoryItem";
import Camera from "@/store/classes/Camera";


const db = new BilliDB("billibox", 3);


const stores = [
    "inventory",
    "brands",
    "conditions",
    "buildtypes",
    "manufacturers",
    "medias",
    "mounts",
    "sets"
];


const state = {
    isInitialized: false,
    inventory: [],
    brands: [],
    manufacturers: [],
    conditions: [],
    medias: [],
    mounts: [],
    sets: [],
    buildtypes: []

};

const getters = {
    inventory: (state:any) => {
      return state.inventory
    },

    cameras: (state: any) => {
       return state.inventory.find((item:InventoryItem) => item.item_type ==="camera");
    },
    camera: (state: any) => (id: any) => state.inventory.find((item: any) => item.item_type==="camera" &&item.id === id),
    activeItem: (state: any) => state.inventory.find((item:any) => item.active ),

    lenses: (state: any) => {
        return state.inventory.find((item:InventoryItem) => item.item_type ==="lens");
    },
    lens: (state: any) => (id: any) => state.inventory.find((item: any) => item.item_type==="lens" &&item.id === id),

    accessories: (state: any) => {
        return state.inventory.find((item:InventoryItem) => item.item_type ==="accessoire");
    },
    accessoire: (state: any) => (id: any) => state.inventory.find((item: any) => item.item_type==="accessoire" &&item.id === id),

    brands: (state: any) => state.brands,
    brand: (state: any) => (id: any) => state.brands.find((brand: any) => brand.key === id),

    manufacturers: (state: any) => state.manufacturers,
    manufacturer: (state: any) => (id: any) => state.manufacturers.find((item: any) => item.key === id),

    mounts: (state: any) => state.mounts,
    mount: (state: any) => (id: any) => state.mounts.find((item: any) => item.key === id),


    conditions: (state: any) => state.conditions,
    condition: (state: any) => (id: any) => state.conditions.find((condition: any) => condition.key === id),

    medias: (state: any) => state.medias,
    media: (state: any) => (id: any) => state.medias.find((media: any) => media.key === id),

    buildtypes: (state: any) => state.buildtypes,
    buildtype: (state: any) => (id: any) => state.buildtypes.find((buildtype: any) => buildtype.key === id),

    isInitialized: (state:any ) => state.isInitialized,

};
const mutations = {
    setInventory(state:any, payload:any) {
        console.log("mutation.setInventory");
        payload.map((item:InventoryItem) => {
            state.inventory.push({
                ...item,
                edit: false,
                active: false
            })
        })
        console.log("STORE", state.inventory.length);
    },

    storeCamera(sate:any, payload:any) {
        const objCamera = state.inventory.find((camera:Camera) => camera.id === payload.id);
        if(objCamera) {
            payload.edit =false;
            objCamera.name = payload.name;
        } else {
            payload.edit = false;
            state.inventory.push(payload);
        }
        return payload.id;
    },

    setItemActiveState(state:any, payload:any) {
        console.log("mutation.setItemActiveState");
        state.inventory.map((item:InventoryItem) => item.active= false);
        const objItem = state.inventory.find((itemId:InventoryItem) => itemId.id === payload.id);
        objItem.active = payload.active;
        console.log("Result of mutation", objItem);
    },

    setItemEditState(state:any, payload:any) {
        state.inventory.map((item:InventoryItem) => item.edit= false);
        const objItem = state.inventory.find((camera:any) => camera.id === payload.id);
        objItem.edit = payload.edit;
    },

    setNameKeys(state:any, payload:any) {
        const field = payload.namekey_type;
        state[field] = payload.values;
    },

    setInitialized(state:any, payload:boolean) {
        state.isInitialized = payload;
    }
};
const actions = {

    fetchNameKeys(context:ActionContext<any, any>, payload) {
        const field =payload.namekey_type;
        db.getItems(field).then((items) => {
            context.commit("setNameKeys", {
                namekey_type: field,
                values:items
            });
        })
    },

    fetchInventory(context:ActionContext<any, any>, payload) {
        console.log("action.fetchInventory")
        db.getItems("inventory").then((items) => {
            context.commit("setInventory", items);
        })
    },

    setItemActiveState(context:ActionContext<any, any>, payload:any) {
        context.commit("setItemActiveState", {id: payload.id, active:payload.active});

    },

    setItemEditState(context:ActionContext<any, any>, payload:any) {
        console.log("action.setItemEditState", payload)
        context.commit("setItemEditState", {id: payload.id, edit:payload.edit});

    },

    storeCamera(context:ActionContext<any, any>, payload:any) {
       // console.log("action.storeCamera", payload)

        if(!payload.id) payload.id = uuidv4();
        //console.log(" ... with id ", payload.id);

        //console.log("Try to save to indexDB");
        db.saveItem(payload , "inventory").then( (res) => {
            console.log("DB-Save resolved", res )}
        )


        context.commit("storeCamera", payload);
        context.commit("setCameraActiveState", {cameraId:payload.id, active:true})

    },

    loadDataFromLocalDB(context:ActionContext<any, any>) {
        console.log("action.loadDataFromLocalDB")
        const storeActions = [];
        for (const store of stores) {
            if(store ==="inventory") {
                storeActions.push(context.dispatch("fetchInventory"));
            }
            else {
                storeActions.push(context.dispatch("fetchNameKeys", {namekey_type:store}));
            }
        }
        Promise.all(storeActions).then( () => {
            context.commit("setInitialized", true);
        });

    },

    initialize(context:ActionContext<any, any>, payload) {

        db.getDb().then( () => {
            const promises = [];
            const seedVersion = localStorage.getItem("SEED_VERSION") || 0;
            if(payload.seed && seedVersion < SEED_VERSION ) {
                console.log("SEEDING database");
                for (const store of stores) {
                    promises.push(db.saveItems(seed[store], store));
                }
                localStorage.setItem("SEED_VERSION", seed.SEED_VERSION.toString())
            } else {
                console.log("SEEDING skipped");
                context.dispatch("loadDataFromLocalDB")
                    .then(() => console.log("Store Data loaded"))
                    .catch((e) => console.log("Error loading StoreData", e));

            }
        });
    }
};

const modules = {};

const inventoryModule = {
    state,
    getters,
    mutations,
    actions,
    modules
}

export default inventoryModule